#include <stdio.h>
#include <curses.h>
#include <unistd.h>
#include "lcm.h"
#include <pthread.h>


pthread_mutex_t mutex;
bool inited=false;
void init();

#define delay(val) usleep(1000*val)

int getKeyLcm()
{ 
  if(!inited) {
    init();
  }
  pthread_mutex_lock(&mutex);
  unsigned char data;
  data = 0xFE;
  writeSerial(&data, 1); //request
  data = 0x0B;
  writeSerial(&data, 1);
  delay(10);
  readSerial(&data, 1);
  pthread_mutex_unlock(&mutex);
  switch(data)
  {
    case 0x0E:
      return UP;
    case 0x0D:
      return RIGHT;
    case 0x0B:
      return LEFT;
    case 0x07:
      return DOWN;
    default:
      return NONE;
  }
}

int displayLcm(bool mode)
{
  if(!inited) {
    init();
  }
  pthread_mutex_lock(&mutex);
  unsigned char data;
  data = 0xFE;
  writeSerial(&data, 1); //request
  if(mode == true)
    data = 0x01;
  else
    data = 0x02;
  writeSerial(&data, 1);
  pthread_mutex_unlock(&mutex);
  return 0;
}

int setPositionLcm(int row, int column)
{
  if(!inited) {
    init();
  }
  if((row != 0) && (row!=1))
    return -1;
  if((column < 0) && (column>15))
    return -1;
  pthread_mutex_lock(&mutex);
  unsigned char data;
  data = 0xFE;
  writeSerial(&data, 1); //request
  data = 0x09;
  writeSerial(&data, 1);
  data = 0x40 * (unsigned char)row + (unsigned char)column;
  writeSerial(&data, 1);
  pthread_mutex_unlock(&mutex);
  return 0;
}

int getPositionLcm(int* row, int* column)
{
  if(!inited) {
    init();
  }
  pthread_mutex_lock(&mutex);
  unsigned char data;
  data = 0xFE;
  writeSerial(&data, 1); //request
  data = 0x0A;
  writeSerial(&data, 1);
  delay(10);
  readSerial(&data, 1);
  if(data >= 0x40)
    *row = 1;
  else
    *row = 0;
  *column = data & 0x0F;
  pthread_mutex_unlock(&mutex);
  return 0;
}

int cursorLcm(bool mode)
{
  if(!inited) {
    init();
  }
  pthread_mutex_lock(&mutex);
  unsigned char data;
  data = 0xFE;
  writeSerial(&data, 1); //request
  if( mode == true )
    data = 0x05;
  else
    data = 0x06;
  writeSerial(&data, 1);
  pthread_mutex_unlock(&mutex);
  return 0;
}

int cursorActionLcm(int type)
{
  if(!inited) {
    init();
  }
  unsigned char data;
  int row, column;
  getPositionLcm(&row, &column);
  switch(type)
  {
    case HOME:
      setPositionLcm(0, 0);
      break;
    case MOVERIGHT:
      if(column < 15)
        column ++;
      setPositionLcm(row, column);
      break;
    case MOVELEFT:
      if(column > 0)
        column --;
      setPositionLcm(row, column);
      break;
    case MOVEBACK:
      if(column > 0)
        column --; 
      printf("row = %d, column = %d\n",row,column);
      setPositionLcm(row, column);
      data = ' ';
      showLcm(1, &data );
      setPositionLcm(row, column);
  }
  return 0;
}

int clrscrLcm()
{
  if(!inited) {
    init();
  }
  pthread_mutex_lock(&mutex);
  unsigned char data;
  data = 0xFE;
  writeSerial(&data, 1); //request
  data = 0x03;
  writeSerial(&data, 1);
  pthread_mutex_unlock(&mutex);
  return 0;
}

int showLcm(int length, unsigned char* info)
{
  if(!inited) {
    init();
  }
  unsigned char data;
  int row, column;
  int i;
  getPositionLcm(&row, &column);
  if(length + column > 15)
    length = 16 - column;
  pthread_mutex_lock(&mutex);
  for(i = 0; i < length; i++) {
    data = *(info + i);
    writeSerial(&data, 1); 
  }
  pthread_mutex_unlock(&mutex);
  if(column + length >=16)
    setPositionLcm(row, 15);
  return 0;
}

void init()
{
  unsigned char start[16]={0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0xfe,0x01,0x02,0x03,0x04,0x05,0x06,0x07};
  int i;
  unsigned char data;
  //init magic words
  initSerialPort(2);
  writeSerial(start, 16);
  pthread_mutex_init(&mutex, NULL);
  inited = true;

}